@@ -6,18 +6,21 @@ Let's make that as easy as possible.
 """
 from _version import __version__
 
-import collections
 import contextlib
 import email
+from email import encoders
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
+from email.mime.audio import MIMEAudio
+from email.mime.base import MIMEBase
+from email.mime.image import MIMEImage
 import itertools
+import mimetypes
 import smtplib
 import types
 
 import ffs
 from ffs.contrib import mold
-import jinja2
 
 __all__ = [
     '__version__',
@@ -53,6 +56,58 @@ def _stringlist(*args):
     """
     return list(itertools.chain.from_iterable(itertools.repeat(x,1) if stringy(x) else x for x in args if x))
 
+
+class Attachment(object):
+    """
+    A file we're attaching to an email.
+    """
+    def __init__(self, path):
+        self.path = ffs.Path(path)
+
+    def as_msg(self):
+        """
+        Convert ourself to be a message part of the appropriate
+        MIME type.
+
+        Return: MIMEBase
+        Exceptions: None
+        """
+        # Based upon http://docs.python.org/2/library/email-examples.html
+        # with minimal tweaking
+
+        # Guess the content type based on the file's extension.  Encoding
+        # will be ignored, although we should check for simple things like
+        # gzip'd or compressed files.
+        ctype, encoding = mimetypes.guess_type(str(self.path))
+        if ctype is None or encoding is not None:
+            # No guess could be made, or the file is encoded (compressed), so
+            # use a generic bag-of-bits type.
+            ctype = 'application/octet-stream'
+        maintype, subtype = ctype.split('/', 1)
+        if maintype == 'text':
+            # Note: we should handle calculating the charset
+            msg = MIMEText(self.path.read(), _subtype=subtype)
+        elif maintype == 'image':
+            fp = self.path.open('rb')
+            msg = MIMEImage(fp.read(), _subtype=subtype)
+            fp.close()
+        elif maintype == 'audio':
+            fp = self.path.open('rb')
+            msg = MIMEAudio(fp.read(), _subtype=subtype)
+            fp.close()
+        else:
+            fp = self.path.open('rb')
+            msg = MIMEBase(maintype, subtype)
+            msg.set_payload(fp.read())
+            fp.close()
+            # Encode the payload using Base64
+            encoders.encode_base64(msg)
+
+        filename = str(self.path[-1])
+        msg.add_header('Content-Disposition', 'attachment', filename=filename)
+        return msg
+
+
 class BaseMailer(object):
     """
     Mailers either handle the construction and delivery of message
@@ -99,7 +154,8 @@ class BaseSMTPMailer(BaseMailer):
     Construct the message
     """
 
-    def send(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None):
+    def send(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None,
+             replyto=None, attach=None):
         """
         Send the message.
 
@@ -116,20 +172,26 @@ class BaseSMTPMailer(BaseMailer):
         - `html`: str
         - `cc`: str or [str]
         - `bcc`: str or [str]
+        - `replyto`: str
+        - `attach`: str or [str]
 
         Return: None
         Exceptions: NoContentError
         """
         self.sanity_check(sender, to, subject, plain=plain, html=html)
         # Create message container - the correct MIME type is multipart/alternative.
-        msg = MIMEMultipart('alternative')
+        msg = MIMEMultipart('mixed')
         msg['Subject'] = u(subject)
         msg['From']    = u(sender)
         msg['To']      = self.tolist(to)
-        msg['Cc']      = self.tolist(cc)
+        if cc:
+            msg['Cc']      = self.tolist(cc)
 
         recipients = _stringlist(to, cc, bcc)
 
+        if replyto:
+            msg.add_header('reply-to', replyto)
+
         # Attach parts into message container.
         # According to RFC 2046, the last part of a multipart message, in this case
         # the HTML message, is best and preferred.
@@ -138,6 +200,11 @@ class BaseSMTPMailer(BaseMailer):
         if html:
             msg.attach(MIMEText(u(html), 'html'))
 
+        # Deal with attachments.
+        if attach:
+            for p in _stringlist(attach):
+                msg.attach(Attachment(p).as_msg())
+
         self.deliver(msg, recipients)
 
 
@@ -167,7 +234,6 @@ class SMTPMailer(BaseSMTPMailer):
         s = smtplib.SMTP(self.host, self.port)
         # sendmail function takes 3 arguments: sender's address, recipient's address
         # and message to send - here it is sent as one string.
-
         s.sendmail(message['From'], to, message.as_string())
         s.quit()
         return
@@ -212,7 +278,8 @@ class DjangoMailer(BaseMailer):
     email settings etc etc
     """
 
-    def send(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None):
+    def send(self, sender, to, subject, plain=None, html=None, cc=None, bcc=None,
+             attach=None, replyto=None):
         """
         Send the message.
 
@@ -227,16 +294,26 @@ class DjangoMailer(BaseMailer):
         - `subject`: str
         - `plain`: str
         - `html`: str
+        - `attach`: str or iterable of str
+        - `replyto`: str
 
         Return: None
         Exceptions: NoContentError
         """
-        if cc or bcc:
-            raise NotImplementedError('Cc & Bcc not implemented for Django yet!')
+        headers = {}
+        if attach:
+            raise NotImplementedError('Attachments not implemented for Django yet!')
+        if replyto:
+            headers['Reply-To'] = replyto
 
         self.sanity_check(sender, to, subject, plain=plain, html=html,
                           cc=cc, bcc=bcc)
 
+        if not cc:
+            cc = []
+        if not bcc:
+            bcc = []
+
         # This comes straight from the docs at
         # https://docs.djangoproject.com/en/dev/topics/email/
         from django.core.mail import EmailMultiAlternatives
@@ -244,7 +321,8 @@ class DjangoMailer(BaseMailer):
         if not plain:
             plain = ''
 
-        msg = EmailMultiAlternatives(u(subject), u(plain), u(sender), _stringlist(to))
+        msg = EmailMultiAlternatives(u(subject), u(plain), u(sender), _stringlist(to),
+                                     bcc=bcc, cc=cc, headers=headers)
 
         if html:
             msg.attach_alternative(u(html), "text/html")
@@ -304,7 +382,7 @@ class BasePostman(object):
         """
         return self._find_tpl(name, extension='.txt'), self._find_tpl(name, extension='.html')
 
-    def _send(self, sender, to, subject, message, cc=None, bcc=None):
+    def _send(self, sender, to, subject, message, cc=None, bcc=None, attach=None, replyto=None):
         """
         Send a Letter (MESSAGE) from SENDER to TO, with the subject SUBJECT
 
@@ -315,16 +393,17 @@ class BasePostman(object):
         - `message`: unicode
         - `cc`: str or [str]
         - `bcc`: str or [str]
+        ` `replyto`: str
 
         Return: None
         Exceptions: None
         """
-        self.mailer.send(sender, to, subject, plain=message, cc=cc, bcc=bcc)
+        self.mailer.send(sender, to, subject, plain=message, cc=cc, bcc=bcc, attach=attach, replyto=replyto)
         return
 
     send = _send
 
-    def _sendtpl(self, sender, to, subject, cc=None, bcc=None, **kwargs):
+    def _sendtpl(self, sender, to, subject, cc=None, bcc=None, attach=None, replyto=None, **kwargs):
         """
         Send a Letter from SENDER to TO, with the subject SUBJECT.
         Use the current template, with KWARGS as the context.
@@ -335,15 +414,15 @@ class BasePostman(object):
         - `subject`: unicode
         - `cc`: str or [str]
         - `bcc`: str or [str]
+        - `replyto`: str
         - `**kwargs`: objects
 
         Return: None
         Exceptions: None
         """
-        # message = mold.cast(self._activetpl, **kwargs)
-        # self._send(sender, to, subject, message)
         plain, html = self.body(**kwargs)
-        self.mailer.send(sender, to, subject, plain=plain, html=html, cc=cc, bcc=bcc)
+        self.mailer.send(sender, to, subject, plain=plain, html=html, cc=cc, bcc=bcc,
+                         replyto=replyto, attach=attach)
         return
 
     def body(self, **kwargs):
@@ -462,16 +541,20 @@ class Letter(object):
     @classmethod
     def send(klass):
         to = klass.To
+        subject = getattr(klass, 'Subject', '')
+
         if isinstance(to, types.StringTypes):
             to = [to]
         if getattr(klass, 'Body', None):
             klass.Postie.send(
                 klass.From,
                 to,
-                klass.Subject,
+                subject,
                 klass.Body,
                 cc=getattr(klass, 'Cc', None),
                 bcc=getattr(klass, 'Bcc', None),
+                replyto=getattr(klass, 'ReplyTo', None),
+                attach=getattr(klass, 'Attach', None),
                 )
             return
 
@@ -479,10 +562,12 @@ class Letter(object):
             klass.Postie.send(
                 klass.From,
                 to,
-                klass.Subject,
+                subject,
                 cc=getattr(klass, 'Cc', None),
                 bcc=getattr(klass, 'Bcc', None),
-                **klass.Context
+                replyto=getattr(klass, 'ReplyTo', None),
+                attach=getattr(klass, 'Attach', None),
+                **getattr(klass, 'Context', {})
                 )
 
 """
